/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   The GPIB implementation for spectral analyzer is initially writen
 *   by Cecilia Morras (on LavWindows).
 * 
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO:
 *        !) Comunicacion con el analizador espectral.
 *        *) llaves de RF via paralelo:
 *           ! falta descripcion de hardware.
 *           ! falta implementacion.
 *           - falta probarlo.
 *        *) comunicacion con el rotador:
 *           ! falta descripcion de hardware.
 *           ! falta implementacion.
 *           - falta probarlo.
 *           - Grilllado para el rotador (testing).
 *           - Azimuth en (double) para el rotador (???).
 *        !) Version.
 *        *) Estructura de directorios para los archivos.
 *
 *
 **/

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <getopt.h>
#include <syslog.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/file.h>

#include "version.h"
#include "my_stdio.h"
#include "toska.h"
#include "observ_table.h"
#include "observation.h"
#include "analyzer.h"
#include "switch_rf.h"
#include "rotator.h"

/* Flags: */
int daemon_flag       = 0; /* 1 */
int rotator_flag = 1;

const char *name_definition = {"Tiny Observatory for SKA"};
const char *help_text[] ={
#       ifdef VERBOSE_MODE
	"  -v\t\t\t- verbose mode (incremental).",
#       endif
	"  -n, --nodaemon\t- no run as a daemon.",
	"  -l,[--log]\t\t- write events to syslog.",
	"  -d\t\t\t- set data directoy (def.: "DATA_DIR")",
	"  -f\t\t\t- set modes definition file (def.: "MODES_FILE")",
	"  -s\t\t\t- set sequence file (def.: "SEQUENCE_FILE")",
	"  --swrf\t\t- set rf switch port in exadecimal.",
	"  --norot\t\t- Ignored rotator.",
	"  -V\t\t\t- show program version.",
	"  --help\t\t- this help !!!.",
	""
};

void version(char *program_name)
{
	fprintf(stderr,"%s: ", name_definition);
	fprintf(stderr,"%s version %s\n", program_name, VERSION);
}

void usage(char *program_name)
{
	version(program_name);
	fprintf(stderr,
#               ifdef VERBOSE_MODE
		"Usage: \t%s [-vnlhdfs][--nodaemon][--log][--help]\n",
#                else
		"Usage: \t%s [-nlhdfs][--nodaemon][--log][--help]\n",
#               endif
		program_name);
}


void help()
{
	int i=0;

	while((char) *help_text[i] != 0)
		fprintf(stderr,"%s\n",help_text[i++]);
	fprintf(stderr,"\n");
}


int main(int argc, char **argv)
{
	int an_dev;
	int i;
	char *data_dir=DATA_DIR;
	char *modes_file = MODES_FILE;
	char *sequence_file = SEQUENCE_FILE;
	char *program_name;
	int switch_rf_port = SWITCH_RF_PORT;
	int pid;
	struct modes *obs, *obsp;
	struct sequence *seq, *seqp;
	struct obs_setup *s;
	char *cp;
	int op;
	static struct option  const  longopts[] =
	{
		{"help", 0, 0, 101},
		{"nodaemon", 0, 0, 'n'},
		{"log", 0, 0, 'l'},
		{"swrf", 1, 0, 201},
		{"norot", 0, 0, 202},
		{0,  0,  0,  0}
	};

	/* who is my name? */
	if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
	else program_name  = argv[0];
  
	verbose_flag = 0;
	opterr =  0;
	while  ((op = getopt_long(argc, argv, "Vvnld:f:s:",
				  longopts, 0)) != EOF)
		switch( op ){
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
		case 101:
			usage(program_name);
			help();
			exit(-1);
		case 'n':
			daemon_flag = 0;
			break;
		case 'l':
			syslog_flag = 1;
			break;
		case 'd':
			data_dir = optarg;
			break;
		case 'f':
			modes_file = optarg;
			break;
		case 201:
			switch_rf_port = strtol(optarg, (char **)NULL, 16);
			break;
		case 202:
			rotator_flag = 0;
			break;
		case 's':
			sequence_file = optarg;
			break;
		case 'V':
			version(program_name);
			exit(-1);
			break;
		default:
			usage(program_name);
			exit(-1);
		}
	
	if(daemon_flag){
		/* Ok, a daemond */
		verbose_flag = 0;
		if ( (pid=fork()) != 0 ) {
			exit(0);
		}
	}
	
	if(syslog_flag){
		openlog(program_name, LOG_PID, LOG_DAEMON );
		SYSLOG(LOG_NOTICE, "Startup succeeded.");
	}

	PVERB(PINFO,"Reading modes file: %s\n", modes_file);
	obs = obsp = get_modes_table(modes_file);
	if( obs == NULL){
		SYSLOG(LOG_ERR, "Error opening modes file: %s.",
		       modes_file );
		exit(1);
	}
	PVERB(PINFO,"Reading sequence file: %s\n", sequence_file);
	seq = seqp = get_sequence_table(sequence_file);
	if( seq == NULL){
		SYSLOG(LOG_ERR, "Error opening sequence file: %s.",
		       sequence_file );
		exit(1);
	}

	s = (struct obs_setup *) malloc(sizeof(struct obs_setup));
	if( s == NULL){
		SYSLOG(LOG_ERR, "Canot alocated memory (struct obs_setup).");
		exit(1);
	}
	s->obs = obs;
	s->data_dir = data_dir;
	s->azimuth = -1;
	s->naxis1 = -1;
	s->polarization = 0;

	i = 0;
	do {
	  an_dev = open_analyzer(AN_PADDR);
	  if (an_dev < 0) {
	    sleep(1);
	  }
	  i++;
	}while(an_dev < 0 && i < AN_RETRY_TIME_OUT);

	if (rotator_flag) {
		if ((s->fd_rot = rotator_setup(ROTATOR_DEVICE)) < 0) {
			SYSLOG(LOG_CRIT, "Canot opend rotator device (%s).",
			       ROTATOR_DEVICE);
		}
	}

	if (antenna_switchs_open(switch_rf_port) != 0) {
		SYSLOG(LOG_CRIT, "Cannot acces rf switch port.");
#ifndef DEMO_MODE
		exit(1);
#endif
	}

	if (an_dev >= 0) {
		while (seqp != NULL ) {
			s->obs = obs;
			s->mode = seqp->mode;
			s->azimuth = seqp->azimuth;
			s->polarization = seqp->polarization;
			s->partition = seqp->partition;
			observ_mode(an_dev, s);
			if(!seqp->restart)
				seqp = seqp->next;
			else
				seqp = seq;
		}
	}
	antenna_switchs_close();
	if (rotator_flag)
		rotator_close(s->fd_rot);
	free_modes_table(obs);
	free_sequence_table(seq);
	free(s);

	return 0;
}
